#include <FastLED.h>

#define LED_PIN_LEFT     3
#define LED_PIN_RIGHT    4
#define NUM_LEDS    64
#define CHIPSET     WS2811
#define COLOR_ORDER GRB
CRGBArray<NUM_LEDS> leds[2];

#define BRIGHTNESS  50
#define PEAK_TIME   2000

int leftChannel = 2; //analog pin
int rightChannel = 1; //analog pin

int volume = 1023; //0 to 1023 - high value = lower volume
int addLedsConstant = 0 ; //how fast are LEDS added to bar
int removeLedsTimeConstant = 6; //how fast are LEDS removed from bar
int middleColorLED = 25;
int peakpos[2];
long peaktime[2];

int numLedsToLight[2];
long lastRefreshTime[2];

void setup() {
  FastLED.addLeds<CHIPSET, LED_PIN_LEFT, COLOR_ORDER>(leds[0], NUM_LEDS).setCorrection( TypicalSMD5050 );
  FastLED.addLeds<CHIPSET, LED_PIN_RIGHT, COLOR_ORDER>(leds[1], NUM_LEDS).setCorrection( TypicalSMD5050 );
  FastLED.setBrightness(BRIGHTNESS);
  analogReference(INTERNAL);//ADMUX |= 0xc0; //Internal 1.1V voltage reference with external capacitor at AREF pin
  digitalWrite(10, LOW);
  digitalWrite(12, HIGH);
  digitalWrite(13, LOW);
}

void loop() {
  int i, ledsToLight[2];
  volume = 1023;
  if (digitalRead(11)) {
    volume = 127;
  }
  ledsToLight[0] = map(analogRead(leftChannel), 0, volume, 0, NUM_LEDS);
  ledsToLight[1] = map(analogRead(rightChannel), 0, volume, 0, NUM_LEDS);

  for (i=0; i<2; i++) {
    if (ledsToLight[i] > numLedsToLight[i]) {
      numLedsToLight[i] = ledsToLight[i];
    } else if (ledsToLight[i] < numLedsToLight[i]) {
      if(millis() - lastRefreshTime[i] >= removeLedsTimeConstant) {
        lastRefreshTime[i] += removeLedsTimeConstant;
        numLedsToLight[i] -= 1;
      }
    }
        
    if (numLedsToLight[i] < 1) {
      numLedsToLight[i] = 0;
    }
    if (numLedsToLight[i] >= NUM_LEDS) {
      numLedsToLight[i] = NUM_LEDS-1;
    }
    if (numLedsToLight[i] > peakpos[i]) {
      peakpos[i] = numLedsToLight[i];
      peaktime[i] = millis()+PEAK_TIME;
    }
    
  
    for(int led = 0; led < numLedsToLight[i]; led++) {
      if (led < middleColorLED) {
        leds[i][led] = CRGB(map(led, 0, middleColorLED - 1, 0, 255) ,255,0);
      } else {
        leds[i][led] = CRGB(255, map(led, middleColorLED, NUM_LEDS, 255, 0),0);
      }
    }
    for(int led = NUM_LEDS-1; led >= numLedsToLight[i]; led--) {
      leds[i][led] = CRGB(0, 0, 0);
    }
    if (1)
    {
      int led = peakpos[i];
      if (led) {
        if (led < middleColorLED) {
          leds[i][led] = CRGB(map(led, 0, middleColorLED - 1, 0, 255) ,255,0);
        } else {
          leds[i][led] = CRGB(255, map(led, middleColorLED, NUM_LEDS, 255, 0),0);
        }
      }
      if (millis() >= peaktime[i]) {
        peaktime[i] = millis()+PEAK_TIME;
        peakpos[i] = 0;
      }
    }
  }
  FastLED.show();
}
